@@ -6,18 +6,19 @@
 __metaclass__ = type
 
 __all__ = [
-    'GDPWindow'
-    'PluginMixin',
+    'ControllerMixin',
+    'set_file_line',
+    'setup_file_lines_view',
     ]
 
 
 import mimetypes
 import os
 
-import pango as Pango
-import gobject as GObject
-import gtk as Gtk
-import gedit
+from gi.repository import GObject
+from gi.repository import Gio
+from gi.repository import Pango
+from gi.repository import Gtk
 
 
 # Initialise the mimetypes for document type inspection.
@@ -27,69 +28,7 @@ mimetypes.add_type('application/x-zope-page-template', '.pt')
 mimetypes.add_type('text/x-python-doctest', '.doctest')
 
 
-# Signals shared in GDP.
-
-GObject.signal_new(
-    'syntax-error-python', gedit.Document, GObject.SIGNAL_RUN_LAST,
-    GObject.TYPE_NONE, ())
-
-
-GObject.signal_new(
-    'bzr-branch-open', gedit.Window, GObject.SIGNAL_RUN_LAST,
-    GObject.TYPE_NONE, (GObject.TYPE_STRING, ))
-
-
-# Common GDP classes.
-
-class GDPWindow:
-    """Decorate a `GeditWindow` with GDP state"""
-
-    def __init__(self, window, controller, plugin):
-        self.window = window
-        self.controller = controller
-        self.signal_ids = {}
-        self.document = None
-        self.ui_id = None
-        if plugin.action_group_name is None:
-            return
-        self.action_group = Gtk.ActionGroup(name=plugin.action_group_name)
-        self.action_group.set_translation_domain('gedit')
-        self.action_group.add_actions(plugin.actions(controller))
-        manager = self.window.get_ui_manager()
-        manager.insert_action_group(self.action_group, -1)
-        self.ui_id = manager.add_ui_from_string(plugin.menu_xml)
-
-    def deactivate(self):
-        """Deactivate the plugin for the window."""
-        if self.ui_id is None:
-            return
-        manager = self.window.get_ui_manager()
-        manager.remove_ui(self.ui_id)
-        manager.remove_action_group(self.action_group)
-        manager.ensure_update()
-        self.controller.deactivate()
-
-    def connect_signal(self, obj, signal, method):
-        """Connect the signal from the provided object to a method."""
-        if obj is None:
-            return
-        self.signal_ids[signal] = obj.connect(signal, method)
-
-    def disconnect_signal(self, obj, signal):
-        """Disconnect the signal from the provided object."""
-        if obj is None:
-            return
-        if signal in self.signal_ids:
-            obj.disconnect(self.signal_ids[signal])
-            del self.signal_ids[signal]
-
-    @property
-    def active_document(self):
-        """The active document in the window."""
-        self.window.get_active_document()
-
-
-class PluginMixin:
+class ControllerMixin:
     """Provide common features to plugins"""
 
     def deactivate(self):
@@ -98,7 +37,7 @@ class PluginMixin:
 
     @staticmethod
     def is_editable(mime_type):
-        """ Only search mime-types that gedit can open.
+        """ Only search mime-types that Gedit can open.
 
         A fuzzy match of text/ or +xml is good, but some files types are
         unknown or described as application data.
@@ -124,8 +63,10 @@ class PluginMixin:
 
     def is_doc_open(self, uri):
         """Return True if the window already has a document opened for uri."""
+        # get_uri_for_display() can return an abs path?
+        path = uri.replace('file://', '')
         for doc in self.window.get_documents():
-            if doc.get_uri() == uri:
+            if doc.get_uri_for_display() in (uri, path):
                 return True
         return False
 
@@ -136,13 +77,17 @@ class PluginMixin:
         mime_type, charset_ = mimetypes.guess_type(uri)
         if self.is_editable(mime_type):
             jump_to = jump_to or 0
-            self.window.create_tab_from_uri(uri, None, jump_to, False, False)
+            location = Gio.file_new_for_uri(uri)
+            self.window.create_tab_from_location(
+                location, None, jump_to, 0, False, False)
             self.window.get_active_view().scroll_to_cursor()
 
     def activate_open_doc(self, uri, jump_to=None):
         """Activate (or open) a document and jump to the line number."""
         self.open_doc(uri, jump_to)
-        self.window.set_active_tab(self.window.get_tab_from_uri(uri))
+        location = Gio.file_new_for_uri(uri)
+        self.window.set_active_tab(
+            self.window.get_tab_from_location(location))
         if jump_to is not None:
             self.window.get_active_document().goto_line(jump_to)
             self.window.get_active_view().scroll_to_cursor()
@@ -154,7 +99,7 @@ class PluginMixin:
 
     @property
     def text(self):
-        """The text of the active gedit.Document or None."""
+        """The text of the active Gedit.Document or None."""
         document = self.window.get_active_document()
         start_iter = document.get_start_iter()
         end_iter = document.get_end_iter()
@@ -164,7 +109,7 @@ class PluginMixin:
 def set_file_line(column, cell, model, piter, cell_type):
     """Set the value as file or line information."""
     file_path = model.get_value(piter, 0)
-    icon = model.get_value(piter, 1)
+    #icon = model.get_value(piter, 1)
     line_no = model.get_value(piter, 2)
     text = model.get_value(piter, 3)
     if text is None:
@@ -199,7 +144,10 @@ def on_file_lines_row_activated(treeview, path, view_column, plugin):
 
 def on_file_lines_resize(treeview, allocation, column, cell):
     """Set the width of the column to update text wrapping."""
-    new_width = allocation.width - 60
+    # Wrap before the vertical scrollbar to ensure there is never a
+    # horizonal scrollbar.
+    parent_width = treeview.get_parent().get_allocated_width()
+    new_width = parent_width - 100
     if cell.props.wrap_width == new_width:
         return
     cell.props.wrap_width = new_width
@@ -220,7 +168,7 @@ def setup_file_lines_view(file_lines_view, plugin, column_title):
     column = Gtk.TreeViewColumn(column_title)
     # icon.
     cell = Gtk.CellRendererPixbuf()
-    cell.set_property('stock-size', Gtk.ICON_SIZE_MENU)
+    cell.set_property('stock-size', Gtk.IconSize.MENU)
     cell.props.yalign = 0
     column.pack_start(cell, False)
     column.add_attribute(cell, 'icon-name', 1)
@@ -228,13 +176,13 @@ def setup_file_lines_view(file_lines_view, plugin, column_title):
     cell = Gtk.CellRendererText()
     cell.props.yalign = 0
     cell.props.xalign = 1
-    cell.props.alignment = Pango.ALIGN_RIGHT
+    cell.props.alignment = Pango.Alignment.RIGHT
     cell.props.family = 'Monospace'
     column.pack_start(cell, False)
     column.set_cell_data_func(cell, set_file_line, 'line_no')
     # line text.
     cell = Gtk.CellRendererText()
-    cell.props.wrap_mode = Pango.WRAP_WORD
+    cell.props.wrap_mode = Pango.WrapMode.WORD
     cell.props.wrap_width = 310
     column.pack_start(cell, False)
     column.set_cell_data_func(cell, set_file_line, 'text')
