@@ -14,17 +14,19 @@ __all__ = [
     ]
 
 from collections import namedtuple
+from gettext import gettext as _
 import mimetypes
 import os
 import re
+import sre_constants
 import sys
 import threading
 from optparse import OptionParser
 
-import gobject as GObject
-import gtk as Gtk
+from gi.repository import GObject
+from gi.repository import Gtk
 
-from gdp import PluginMixin, setup_file_lines_view
+from gdp import ControllerMixin, setup_file_lines_view
 
 
 find_params = namedtuple(
@@ -55,7 +57,7 @@ def find_files(root_dir, skip_dir_pattern='^[.]', file_pattern='.*'):
             if os.path.islink(file_path):
                 continue
             mime_type, encoding_ = mimetypes.guess_type(file_)
-            if PluginMixin.is_editable(mime_type):
+            if ControllerMixin.is_editable(mime_type):
                 if file_re.match(file_path) is not None:
                     yield file_path, mime_type
 
@@ -97,39 +99,54 @@ class FinderWorker(threading.Thread):
         self.find_params = find_params
         self.callback = callback
         self.substitution = substitution
-        self.theme = Gtk.icon_theme_get_default()
+        self.theme = Gtk.IconTheme.get_default()
+
+    @property
+    def pattern(self):
+        pattern = self.find_params.pattern
+        if not self.find_params.is_re:
+            pattern = re.escape(pattern)
+        if not self.find_params.is_case:
+            pattern = '(?i)%s' % pattern
+        return pattern
 
     def run(self):
-        for summary in find_matches(
-            self.find_params.path, self.find_params.file_pattern,
-            self.find_params.pattern, substitution=self.substitution):
-            file_path = summary['file_path']
-            mime_type = summary['mime_type'] or 'text'
-            mime_type = 'gnome-mime-%s' % mime_type.replace('/', '-')
-            if not self.theme.has_icon(mime_type):
-                mime_type = 'gnome-mime-text'
-            piter = self.treestore.append(
-                None, (file_path, mime_type, 0, None, self.find_params.path))
-            if self.substitution is None:
-                icon = Gtk.STOCK_FIND
-            else:
-                icon = Gtk.STOCK_FIND_AND_REPLACE
-            for line in summary['lines']:
-                self.treestore.append(piter,
-                    (file_path, icon, line['lineno'], line['text'],
-                     self.find_params.path))
-        if self.treestore.get_iter_first() is None:
+        try:
+            for summary in find_matches(
+                self.find_params.path, self.find_params.file_pattern,
+                self.pattern, substitution=self.substitution):
+                file_path = summary['file_path']
+                mime_type = summary['mime_type'] or 'text'
+                mime_type = 'gnome-mime-%s' % mime_type.replace('/', '-')
+                if not self.theme.has_icon(mime_type):
+                    mime_type = 'gnome-mime-text'
+                piter = self.treestore.append(
+                    None,
+                    (file_path, mime_type, 0, None, self.find_params.path))
+                if self.substitution is None:
+                    icon = Gtk.STOCK_FIND
+                else:
+                    icon = Gtk.STOCK_FIND_AND_REPLACE
+                for line in summary['lines']:
+                    self.treestore.append(piter,
+                        (file_path, icon, line['lineno'], line['text'],
+                         self.find_params.path))
+            if self.treestore.get_iter_first() is None:
+                message = 'No matches found'
+                self.treestore.append(
+                    None, (message, 'stock_dialog-info', 0, None, None))
+        except sre_constants.error, e:
+            message = 'Find could not be run: %s' % str(e)
             self.treestore.append(
-                None,
-                ('No matches found', 'stock_dialog-info', 0, None, None))
+                None, (message, 'stock_dialog-info', 0, None, None))
         self.callback()
 
 
-class Finder(PluginMixin):
+class Finder(ControllerMixin):
     """Find and replace content in files."""
 
     WORKING_DIRECTORY = '<Working Directory>'
-    CURRENT_FILE= '<Current File>'
+    CURRENT_FILE = '<Current File>'
     ANY_FILE = '<Any Text File>'
 
     def __init__(self, window):
@@ -142,16 +159,14 @@ class Finder(PluginMixin):
         self.setup_widgets()
         self.find_panel = self.widgets.get_object('find_side_panel')
         panel = window.get_side_panel()
-        icon = Gtk.image_new_from_stock(Gtk.STOCK_FIND, Gtk.ICON_SIZE_MENU)
-        panel.add_item(self.find_panel, 'Find in files', icon)
+        icon = Gtk.Image.new_from_stock(Gtk.STOCK_FIND, Gtk.IconSize.MENU)
+        panel.add_item(self.find_panel, 'gdpfind', 'Find in files', icon)
 
     def setup_widgets(self):
         """Setup the widgets with default data."""
         self.widgets.connect_signals(self.ui_callbacks)
         self.pattern_comboentry = self.widgets.get_object(
             'pattern_comboentry')
-        self.pattern_comboentry.get_child().connect(
-            'activate', self.on_find_in_files)
         self.pattern_comboentry.get_child().set_width_chars(24)
         self.setup_comboentry(self.pattern_comboentry)
         self.path_comboentry = self.widgets.get_object('path_comboentry')
@@ -171,23 +186,27 @@ class Finder(PluginMixin):
         panel.remove_item(self.find_panel)
 
     def setup_comboentry(self, comboentry, default=None):
-        liststore = Gtk.ListStore(GObject.TYPE_STRING)
-        liststore.set_sort_column_id(0, Gtk.SORT_ASCENDING)
+        liststore = Gtk.ListStore.new([GObject.TYPE_STRING])
+        liststore.set_sort_column_id(0, Gtk.SortType.ASCENDING)
         comboentry.set_model(liststore)
-        comboentry.set_text_column(0)
+        comboentry.set_entry_text_column(0)
         if default is not None:
             self.update_comboentry(comboentry, default)
 
-    def update_comboentry(self, comboentry, text):
+    def update_comboentry(self, comboentry, text, set_active=True):
         """Update the match text combobox."""
+        found_index = None
         for index, row in enumerate(iter(comboentry.get_model())):
             if row[0] == text:
-                # The text is already in the list, it just need to be active.
-                comboentry.set_active(index)
-                return
-        comboentry.append_text(text)
-        # Call this again to set the text active.
-        self.update_comboentry(comboentry, text)
+                # The text is already in the list, does it need to be active?
+                found_index = index
+                break
+        if found_index is not None and set_active:
+            comboentry.set_active(found_index)
+        elif found_index is None:
+            comboentry.append_text(text)
+            if set_active:
+                self.update_comboentry(comboentry, text)
 
     @property
     def ui_callbacks(self):
@@ -198,15 +217,15 @@ class Finder(PluginMixin):
             'on_save_results': self.on_save_results,
             }
 
-    def show(self, data):
+    def show(self, action):
         """Show the finder pane."""
         panel = self.window.get_side_panel()
         panel.activate_item(self.find_panel)
         panel.props.visible = True
 
-    def show_replace(self, data):
+    def show_replace(self, action):
         """Show the finder pane and expand replace."""
-        self.show(None)
+        self.show(None, None)
         self.widgets.get_object('actions').activate()
 
     @property
@@ -239,8 +258,9 @@ class Finder(PluginMixin):
         self.update_comboentry(self.pattern_comboentry, pattern)
         return pattern
 
-    def on_file_path_added(self, window, path):
-        self.update_comboentry(self.path_comboentry, path)
+    def on_file_path_added(self, window, new_path):
+        self.update_comboentry(
+            self.path_comboentry, new_path, set_active=False)
 
     def get_find_params(self):
         """Return the find parameters as a tuple."""
@@ -260,11 +280,6 @@ class Finder(PluginMixin):
         pattern = find_params.pattern
         self.file_lines_view.get_column(0).props.title = (
             'Matches for [%s]' % pattern)
-        if not find_params.is_re:
-            pattern = re.escape(pattern)
-        if not find_params.is_case:
-            pattern = '(?i)%s' % pattern
-        base_dir = find_params.path
         find_worker = FinderWorker(
             treestore, find_params, self.on_find_complete, substitution)
         find_worker.run()
@@ -277,14 +292,15 @@ class Finder(PluginMixin):
         """Find, replace, and present the matches."""
         substitution = self.substitution_comboentry.get_active_text() or ''
         self.update_comboentry(self.substitution_comboentry, substitution)
-        response = Gtk.RESPONSE_ACCEPT
+        response = Gtk.ResponseType.ACCEPT
         find_params = self.get_find_params()
         if self.last_find != find_params:
+            dialog_flags = Gtk.DialogFlags
             dialog = Gtk.Dialog(
                 title="Untested replacement", parent=self.window,
-                flags=Gtk.DIALOG_MODAL|Gtk.DIALOG_DESTROY_WITH_PARENT,
-                buttons=(Gtk.STOCK_FIND, Gtk.RESPONSE_REJECT,
-                         Gtk.STOCK_FIND_AND_REPLACE, Gtk.RESPONSE_ACCEPT))
+                flags=dialog_flags.MODAL | dialog_flags.DESTROY_WITH_PARENT,
+                buttons=(Gtk.STOCK_FIND, Gtk.ResponseType.REJECT,
+                         Gtk.STOCK_FIND_AND_REPLACE, Gtk.ResponseType.ACCEPT))
             question = Gtk.Label(
                 _("Do want to test this replacement using Find first?"))
             question.set_alignment(0, 0)
@@ -311,21 +327,21 @@ class Finder(PluginMixin):
             params_summary.show()
             response = dialog.run()
             dialog.destroy()
-        if response == Gtk.RESPONSE_REJECT:
+        if response == Gtk.ResponseType.REJECT:
             self.on_find_in_files()
-        elif response == Gtk.RESPONSE_ACCEPT:
+        elif response == Gtk.ResponseType.ACCEPT:
             self.on_find_in_files(substitution=substitution)
 
     def on_save_results(self, widget=None):
         """Save the search results to a file."""
         dialog = Gtk.FileChooserDialog(
             title="Save find results", parent=self.window,
-            action=Gtk.FILE_CHOOSER_ACTION_SAVE,
-            buttons=(Gtk.STOCK_CANCEL, Gtk.RESPONSE_CANCEL,
-                     Gtk.STOCK_SAVE, Gtk.RESPONSE_ACCEPT))
+            action=Gtk.FileChooserAction.SAVE,
+            buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                     Gtk.STOCK_SAVE, Gtk.ResponseType.ACCEPT))
         dialog.set_do_overwrite_confirmation(True)
         dialog.set_current_name('_find.log')
-        if (dialog.run() == Gtk.RESPONSE_ACCEPT):
+        if (dialog.run() == Gtk.ResponseType.ACCEPT):
             file_name = dialog.get_filename()
             log_text = self.get_results_as_log()
             with open(file_name, 'w') as log_file:
