@@ -7,21 +7,21 @@ __all__ = [
     'Formatter',
     ]
 
+from gettext import gettext as _
 import os
 import re
-
-import gconf as GConf
-import gtk as Gtk
 from textwrap import wrap
-from gettext import gettext as _
+
+from gi.repository import GConf
+from gi.repository import Gtk
 
 from pocketlint.formatdoctest import DoctestReviewer
 from pocketlint.formatcheck import Language, Reporter, UniversalChecker
 
-from gdp import PluginMixin, setup_file_lines_view
+from gdp import ControllerMixin, setup_file_lines_view
 
 
-class Formatter(PluginMixin):
+class Formatter(ControllerMixin):
     """Format Gedit Document and selection text."""
 
     def __init__(self, window):
@@ -49,8 +49,9 @@ class Formatter(PluginMixin):
         self.file_lines_view = other_widgets.get_object('file_lines_view')
         setup_file_lines_view(self.file_lines_view, self, 'Problems')
         panel = window.get_side_panel()
-        icon = Gtk.image_new_from_stock(Gtk.STOCK_INFO, Gtk.ICON_SIZE_MENU)
-        panel.add_item(self.file_lines, 'Check syntax and style', icon)
+        icon = Gtk.Image.new_from_stock(Gtk.STOCK_INFO, Gtk.IconSize.MENU)
+        panel.add_item(
+            self.file_lines, 'gdpformat', 'Check syntax and style', icon)
 
     def deactivate(self):
         """Clean up resources before deactivation."""
@@ -95,35 +96,35 @@ class Formatter(PluginMixin):
         lines = [line.strip() for line in text.splitlines()]
         return ' '.join(lines)
 
-    def single_line(self, data):
+    def single_line(self, action):
         """Format the text as a single line."""
         bounds, text = self._get_bounded_text()
         text = self._single_line(text)
         self._put_bounded_text(bounds, text)
 
-    def newline_ending(self, data):
+    def newline_ending(self, action):
         """Fix the selection's line endings."""
         bounds, text = self._get_bounded_text()
         lines = [line.rstrip() for line in text.splitlines()]
         self._put_bounded_text(bounds, '\n'.join(lines))
 
-    def tabs_to_spaces(self, data):
+    def tabs_to_spaces(self, action):
         """Fix the selection's line endings."""
         bounds, text = self._get_bounded_text()
-        gconf_client = GConf.client_get_default()
+        gconf_client = GConf.Client.get_default()
         tab_size = gconf_client.get_int(
             '/apps/gedit-2/preferences/editor/tabs/tabs_size') or 4
         tab_spaces = ' ' * tab_size
         lines = [line.replace('\t', tab_spaces) for line in text.splitlines()]
         self._put_bounded_text(bounds, '\n'.join(lines))
 
-    def quote_lines(self, data):
+    def quote_lines(self, action):
         """Quote the selected text passage."""
         bounds, text = self._get_bounded_text()
         lines = ['> %s' % line for line in text.splitlines()]
         self._put_bounded_text(bounds, '\n'.join(lines))
 
-    def sort_imports(self, data):
+    def sort_imports(self, action):
         """Sort python imports."""
         bounds, text = self._get_bounded_text()
         padding = self._get_padding(text)
@@ -134,7 +135,7 @@ class Formatter(PluginMixin):
         text = self._wrap_text(', '.join(imports), padding=padding)
         self._put_bounded_text(bounds, text)
 
-    def wrap_selection_list(self, data):
+    def wrap_selection_list(self, action):
         """Wrap long lines and preserve indentation."""
         # This should use the textwrap module.
         paras = []
@@ -193,13 +194,13 @@ class Formatter(PluginMixin):
         paragraph = '\n'.join(lines)
         return paragraph
 
-    def rewrap_text(self, data):
+    def rewrap_text(self, action):
         """Rewrap the paragraph."""
         bounds, text = self._get_bounded_text()
         text = self._wrap_text(text)
         self._put_bounded_text(bounds, text)
 
-    def reformat_css(self, data):
+    def reformat_css(self, action):
         """Reformat the CSS."""
         bounds, text = self._get_bounded_text()
         # Break the text into rules using the trailing brace; the last item
@@ -226,7 +227,7 @@ class Formatter(PluginMixin):
             css.append(trailing_text)
         self._put_bounded_text(bounds, '\n'.join(css))
 
-    def re_replace(self, data):
+    def re_replace(self, action):
         """Replace each line using an re pattern."""
         self._bounds, self._text = self._get_bounded_text()
         self.replace_dialog.show()
@@ -256,13 +257,13 @@ class Formatter(PluginMixin):
         self._put_bounded_text(self._bounds, '\n'.join(lines))
         self.on_replace_quit()
 
-    def show(self, data):
+    def show(self):
         """Show the finder pane."""
         panel = self.window.get_side_panel()
         panel.activate_item(self.file_lines)
         panel.props.visible = True
 
-    def reformat_doctest(self, data):
+    def reformat_doctest(self, action):
         """Reformat the doctest."""
         bounds, text = self._get_bounded_text()
         file_name = self.active_document.get_uri_for_display()
@@ -283,7 +284,7 @@ class Formatter(PluginMixin):
             file_path, text=text, language=language, reporter=reporter)
         checker.check()
 
-    def check_style(self, data, documents=None, quiet=False):
+    def check_style(self, action, documents=None, quiet=False):
         """Check the style and syntax of the active document."""
         self.file_lines_view.get_model().clear()
         if documents is None:
@@ -299,8 +300,8 @@ class Formatter(PluginMixin):
             # Do not interupt the user to say there is nothing to see.
             return
         self.file_lines_view.expand_all()
-        self.show(None)
+        self.show()
 
-    def check_all_style(self, data):
+    def check_all_style(self, action):
         """Check the style and syntax of all open documents."""
         self.check_style(None, documents=self.window.get_documents())
