@@ -21,14 +21,14 @@ from keyword import kwlist
 from pydoc import TextDoc
 from xml.sax import saxutils
 
-import gobject as GObject
-import gtk as Gtk
-import gtksourceview2 as GtkSource
+from gi.repository import GObject
+from gi.repository import Gtk
+from gi.repository import GtkSource
 
-from gdp import PluginMixin
+from gdp import ControllerMixin
 
 
-lang_manager = GtkSource.language_manager_get_default()
+lang_manager = GtkSource.LanguageManager.get_default()
 doctest_language = lang_manager.get_language('doctest')
 
 doctest_pattern = re.compile(
@@ -51,7 +51,9 @@ def get_word(document, word_pattern, end=None):
     # there is be no word before the cursor.
     start_char = start.copy()
     if start_char.backward_char():
-        char = start_char.get_char()
+        # XXX sinzui 2011-01-07: get_char() dies, but get_text works.
+        # char = start_char.get_char()
+        char = start_char.get_text(start)[0]
         if not word_pattern.match(char):
             return (None, start, end)
 
@@ -60,7 +62,9 @@ def get_word(document, word_pattern, end=None):
     count = 0
     peek = start.copy()
     while peek.backward_chars(1):
-        if not word_pattern.match(peek.get_char()):
+        # XXX sinzui 2011-01-07: get_char() dies, but get_text works.
+        char = peek.get_text(start)[0]
+        if not word_pattern.match(char):
             break
         else:
             count += 1
@@ -151,16 +155,20 @@ class DynamicProvider(GObject.GObject, GtkSource.CompletionProvider):
 
     def __init__(self, name, language_id, handler, document):
         GObject.GObject.__init__(self)
-        self.name = name
+        self._name = name
         self.proposals = []
         self.language_id = language_id
         self.handler = handler
         self.document = document
         self.info_widget = None
         self.mark = None
-        theme = Gtk.icon_theme_get_default()
-        w, h = Gtk.icon_size_lookup(Gtk.ICON_SIZE_MENU)
-        self.icon = theme.load_icon(Gtk.STOCK_JUSTIFY_LEFT, w, 0)
+        theme = Gtk.IconTheme.get_default()
+        s, w, h = Gtk.icon_size_lookup(Gtk.IconSize.MENU)
+        try:
+            self.icon = theme.load_icon(Gtk.STOCK_JUSTIFY_LEFT, w, 0)
+        except:
+            # The theme is not complete.
+            self.icon = None
 
     def mark_position(self, it):
         """Create or move the mark of the word start."""
@@ -177,7 +185,7 @@ class DynamicProvider(GObject.GObject, GtkSource.CompletionProvider):
             self.mark_position(start)
         return word
 
-    def do_get_start_iter(self, context, proposal):
+    def do_get_start_iter(self, context, proposal, iter_):
         """See `CompletionProvider`."""
         if not self.mark or self.mark.get_deleted():
             return None
@@ -220,7 +228,7 @@ class DynamicProvider(GObject.GObject, GtkSource.CompletionProvider):
 
     def do_get_name(self):
         """See `CompletionProvider`."""
-        return self.name
+        return self._name
 
     def do_activate_proposal(self, proposal, piter):
         """See `CompletionProvider`."""
@@ -232,12 +240,12 @@ class DynamicProvider(GObject.GObject, GtkSource.CompletionProvider):
 
     def do_get_activation(self):
         """See `CompletionProvider`."""
-        return GtkSource.COMPLETION_ACTIVATION_USER_REQUESTED
+        return GtkSource.CompletionActivation.USER_REQUESTED
 
     def do_get_info_widget(self, proposal):
         """See `DynamicProvider`."""
         if self.info_widget is None:
-            self.info_view = Gtk.Label('')
+            self.info_view = Gtk.Label(label='')
             self.info_view.set_alignment(0.0, 0.0)
             self.info_widget = Gtk.ScrolledWindow()
             self.info_widget.add_with_viewport(self.info_view)
@@ -249,8 +257,6 @@ class DynamicProvider(GObject.GObject, GtkSource.CompletionProvider):
         self.info_view.set_markup(markup)
         self.info_view.show()
         self.info_widget.set_size_request(400, -1)
-        info.set_sizing(400, -1, False, False)
-        info.process_resize()
 
 GObject.type_register(DynamicProposal)
 GObject.type_register(DynamicProvider)
@@ -306,7 +312,7 @@ class BaseSyntaxGenerator:
     @property
     def file_path(self):
         """The path to the file that is the word source."""
-        return self._document.get_uri()
+        return self._document.get_uri_for_display()
 
     @property
     def text(self):
@@ -360,7 +366,11 @@ class MarkupGenerator(BaseSyntaxGenerator):
         if not start_iter.backward_char():
             # The start was at the begining of the doc; no tags were found.
             return self.OUTSIDE
-        char = start_iter.get_char()
+        # XXX sinzui 2011-01-07: get_char() dies, but get_text works.
+        char_iter = start_iter.copy()
+        char_iter.forward_char()
+        char = start_iter.get_text(char_iter)[0]
+        #char = start_iter.get_char()
         if char == '>':
             return self.OUTSIDE
         elif text and text.startswith('/'):
@@ -555,7 +565,6 @@ class PythonSyntaxGenerator(BaseSyntaxGenerator):
         return ''
 
     def _get_dynamic_proposal(self, module, name):
-        info = None
         if module is None:
             identifier = None
         elif type(module) == 'dict':
@@ -565,7 +574,7 @@ class PythonSyntaxGenerator(BaseSyntaxGenerator):
         return PythonProposal(name, info=identifier)
 
 
-class SyntaxController(PluginMixin):
+class SyntaxController(ControllerMixin):
     """This class manages the gedit.View's interaction with the SyntaxView."""
 
     def __init__(self, window):
@@ -582,10 +591,10 @@ class SyntaxController(PluginMixin):
     def set_view(self, view, is_reset=False):
         """Set the view to be controlled.
 
-        Installs signal handlers for the view. Calling document.get_uri()
-        self.set_view(None) will effectively remove all the control from
-        the current view. when is_reset is True, the current view's
-        signals will be reset.
+        Installs signal handlers for the view. Calling
+        document.get_uri_for_display() self.set_view(None) will effectively
+        remove all the control from the current view. when is_reset is True,
+        the current view's signals will be reset.
         """
         if view is self.view and not is_reset:
             return
@@ -608,7 +617,7 @@ class SyntaxController(PluginMixin):
             obj.disconnect(self.signal_ids[signal])
             del self.signal_ids[signal]
 
-    def show_syntax_view(self, widget=None):
+    def show_syntax_view(self, widget=None, data=None):
         """Show the SyntaxView widget."""
         if not self.view.get_editable():
             return
@@ -624,7 +633,7 @@ class SyntaxController(PluginMixin):
         self.provider = DynamicProvider(
             title, language_id, self.on_proposal_activated, document)
         self.completion.show(
-            [self.provider], self.completion.create_context())
+            [self.provider], self.completion.create_context(None))
 
     def get_word_prefix(self, document):
         """Return a 3-tuple of the word fragment before the cursor.
