@@ -11,18 +11,25 @@ __all__ = [
 
 from gettext import gettext as _
 
-import gedit
+from gi.repository import GObject
 
-from gdp import GDPWindow
+from gdpbase import (
+    GDPPluginMixin,
+    Gedit,
+    )
 from gdp.bzr import BzrProject
 
 
-class BazaarProjectPlugin(gedit.Plugin):
-    """Plugin for formatting code."""
+class BazaarProjectPlugin(GDPPluginMixin,
+                          GObject.Object, Gedit.WindowActivatable):
+    """Plugin for working with source code projects."""
+    __gtype_name__ = "GDPBazaarProjectPlugin"
+    window = GObject.property(type=GObject.Object)
 
-    action_group_name = 'GDPProjectActions'
-    menu_path = '/MenuBar/ProjectMenu/ProjectOps_1'
-    menu_xml = """
+    CONTROLLER_CLASS = BzrProject
+    ACTION_GROUP_NAME = 'GDPProjectActions'
+    MENU_PATH = '/MenuBar/ProjectMenu/ProjectOps_1'
+    MENU_XML = """
         <ui>
           <menubar name="MenuBar">
             <menu action="ProjectMenu">
@@ -82,10 +89,7 @@ class BazaarProjectPlugin(gedit.Plugin):
         ]
 
     def actions(self, bzr):
-        """Return a list of action tuples.
-
-        (name, stock_id, label, accelerator, tooltip, callback)
-        """
+        """See `GDPPluginMixin`"""
         return  [
             ('ProjectMenu', None, _('_Project'), None, None, None),
             ('OpenUncommittedFiles', None, _("Open _uncommitted files"), None,
@@ -142,7 +146,6 @@ class BazaarProjectPlugin(gedit.Plugin):
                 None,
                 _("Mail or create a merge-directive for submitting changes."),
                 bzr.send_merge),
-
             ('InitBranch', None, _("_Initialise branch"),
                 None, _("Make a directory into a versioned branch."),
                 bzr.initialise_branch),
@@ -159,46 +162,46 @@ class BazaarProjectPlugin(gedit.Plugin):
 
     def __init__(self):
         """Initialize the plugin for the Gedit application."""
-        gedit.Plugin.__init__(self)
-        self.windows = {}
+        GObject.Object.__init__(self)
+        self.controller = None
 
-    def activate(self, window):
+    def do_activate(self):
         """Activate the plugin in the current top-level window.
 
         Add 'Project' to the main menu and create a BzrProject.
         """
-        self.windows[window] = GDPWindow(window, BzrProject(window), self)
+        self.activate()
         # Moved the menu to a less surprising position.
-        manager = window.get_ui_manager()
+        manager = self.window.get_ui_manager()
         menubar = manager.get_widget('/MenuBar')
         project_menu = manager.get_widget('/MenuBar/ProjectMenu')
         menubar.remove(project_menu)
         menubar.insert(project_menu, 5)
+        self.do_update_state()
 
-    def deactivate(self, window):
+    def do_deactivate(self):
         """Deactivate the plugin in the current top-level window."""
-        self.windows[window].deactivate()
-        del self.windows[window]
+        self.deactivate()
 
-    def update_ui(self, window):
+    def do_update_state(self):
         """Toggle the plugin's sensativity in the top-level window."""
-        gdp_window = self.windows[window]
-        gdp_window.controller.set_working_tree()
-        if gdp_window.controller.working_tree is None:
-            self.toggle_tree_menus(gdp_window, False)
+        self.controller.set_working_tree()
+        if self.controller.working_tree is None:
+            self.toggle_tree_menus(False)
         else:
-            self.toggle_tree_menus(gdp_window, True)
-            gdp_window.window.emit(
-                'bzr-branch-open', gdp_window.controller.working_tree.basedir)
+            self.toggle_tree_menus(True)
+            self.window.emit(
+                'bzr-branch-open',
+                self.controller.working_tree.basedir)
 
-    def toggle_tree_menus(self, gdp_window, sensitive):
+    def toggle_tree_menus(self, sensitive):
         """Enable or disable the menu items that require a working tree."""
-        manager = gdp_window.window.get_ui_manager()
+        manager = self.window.get_ui_manager()
         for name in self.tree_actions:
-            path = '%s/%s' % (self.menu_path, name)
+            path = '%s/%s' % (self.MENU_PATH, name)
             manager.get_action(path).props.sensitive = sensitive
-        if not gdp_window.controller.has_bzr_gtk:
+        if not self.controller.has_bzr_gtk:
             sensitive = False
         for name in self.bzr_gtk_actions:
-            path = '%s/%s' % (self.menu_path, name)
+            path = '%s/%s' % (self.MENU_PATH, name)
             manager.get_action(path).props.sensitive = sensitive
