@@ -12,19 +12,23 @@ __all__ = [
 
 from gettext import gettext as _
 
-import gedit
+from gi.repository import GObject
 
-from gdp import GDPWindow
+from gdpbase import (
+    GDPPluginMixin,
+    Gedit,
+    )
 from gdp.find import Finder
 
 
-class FindPlugin(gedit.Plugin):
+class FindPlugin(GDPPluginMixin, GObject.Object, Gedit.WindowActivatable):
     """Find matching text in multiple files plugin."""
-    # This is a new-style class that call and old-style __init__().
-    # pylint: disable-msg=W0233
+    __gtype_name__ = "GDPFindPlugin"
+    window = GObject.property(type=GObject.Object)
 
-    action_group_name = 'GDPFindActions'
-    menu_xml = """
+    CONTROLLER_CLASS = Finder
+    ACTION_GROUP_NAME = 'GDPFindActions'
+    MENU_XML = """
         <ui>
           <menubar name="MenuBar">
             <menu name="SearchMenu" action='Search'>
@@ -40,10 +44,7 @@ class FindPlugin(gedit.Plugin):
         """
 
     def actions(self, finder):
-        """Return a list of action tuples.
-
-        (name, stock_id, label, accelerator, tooltip, callback)
-        """
+        """See `GDPPluginMixin`"""
         return [
             ('FindFiles', None, _('Find in files...'),
                 '<Control><Shift>f', _('Fi_nd in files'),
@@ -55,28 +56,27 @@ class FindPlugin(gedit.Plugin):
 
     def __init__(self):
         """Initialize the plugin the whole Gedit application."""
-        gedit.Plugin.__init__(self)
-        self.windows = {}
+        GObject.Object.__init__(self)
+        self.controller = None
 
-    def activate(self, window):
+    def do_activate(self):
         """Activate the plugin in the current top-level window.
 
         Add 'Find in files' to the menu.
         """
-        finder = Finder(window)
-        self.windows[window] = GDPWindow(window, finder, self)
-        self.windows[window].connect_signal(
-            window, 'bzr-branch-open', finder.on_file_path_added)
+        self.activate()
+        self.connect_signal(
+            self.window, 'bzr-branch-open',
+            self.controller.on_file_path_added)
 
-    def deactivate(self, window):
+    def do_deactivate(self):
         """Deactivate the plugin in the current top-level window.
 
         Remove a 'Find in files' to the menu.
         """
-        self.windows[window].deactivate()
-        del self.windows[window]
+        self.deactivate()
 
-    def update_ui(self, window):
+    def do_update_state(self):
         """Toggle the plugin's sensativity in the top-level window.
 
         'Find in files' is always active.
