@@ -11,17 +11,23 @@ __all__ = [
 
 from gettext import gettext as _
 
-import gedit
+from gi.repository import GObject
 
-from gdp import GDPWindow
+from gdpbase import (
+    GDPPluginMixin,
+    Gedit,
+    )
 from gdp.format import Formatter
 
 
-class FormatPlugin(gedit.Plugin):
+class FormatPlugin(GDPPluginMixin, GObject.Object, Gedit.WindowActivatable):
     """Plugin for formatting code."""
+    __gtype_name__ = "GDPFormatPlugin"
+    window = GObject.property(type=GObject.Object)
 
-    action_group_name = 'GDPFormatActions'
-    menu_xml = """
+    CONTROLLER_CLASS = Formatter
+    ACTION_GROUP_NAME = 'GDPFormatActions'
+    MENU_XML = """
         <ui>
           <menubar name="MenuBar">
             <menu name='EditMenu' action='Edit'>
@@ -54,10 +60,7 @@ class FormatPlugin(gedit.Plugin):
         """
 
     def actions(self, formatter):
-        """Return a list of action tuples.
-
-        (name, stock_id, label, accelerator, tooltip, callback)
-        """
+        """See `GDPPluginMixin`"""
         return  [
             ('GDPFormatMenu', None, _('_Format'), None, None, None),
             ('RewrapText', None, _("Rewrap _text"), None,
@@ -98,35 +101,27 @@ class FormatPlugin(gedit.Plugin):
 
     def __init__(self):
         """Initialize the plugin the whole Gedit application."""
-        gedit.Plugin.__init__(self)
-        self.windows = {}
+        GObject.Object.__init__(self)
+        self.controller = None
 
-    def activate(self, window):
+    def do_activate(self):
         """Activate the plugin in the current top-level window.
 
         Add 'Format' to the edit menu and create a Formatter.
         """
-        self.windows[window] = GDPWindow(window, Formatter(window), self)
-        self.on_tab_added_cb = window.connect('tab-added', self.on_tab_added)
+        self.activate()
+        self.connect_signal(self.window, 'tab-added', self.on_tab_added)
 
-    def deactivate(self, window):
+    def do_deactivate(self):
         """Deactivate the plugin in the current top-level window."""
-        window.disconnect(self.on_tab_added_cb)
-        self.windows[window].deactivate()
-        del self.windows[window]
+        self.deactivate()
 
-    def update_ui(self, window):
+    def do_update_state(self):
         """Toggle the plugin's sensativity in the top-level window.
 
         This plugin is always active.
         """
-        gdp_window = self.windows[window]
-        gdp_window.disconnect_signal(
-            gdp_window.document, 'syntax-error-python')
-        gdp_window.document = gdp_window.controller.active_document
-        gdp_window.connect_signal(
-            gdp_window.document, 'syntax-error-python',
-            gdp_window.controller.check_style)
+        pass
 
     def on_tab_added(self, window, tab):
         """Callback method for `tab-added` window signal.
@@ -134,13 +129,14 @@ class FormatPlugin(gedit.Plugin):
         Connects `document-saved` signal.
         """
         document = tab.get_document()
-        if document.get_uri() is not None:
-            controller = self.windows[window].controller
-            document.connect('saved', self.on_document_saved, controller)
+        self.connect_signal(
+            document, 'syntax-error-python', self.controller.check_style)
+        self.connect_signal(
+            document, 'saved', self.on_document_saved)
 
-    def on_document_saved(self, document, user_data, controller):
+    def on_document_saved(self, document, user_data):
         """Callback method for `document-saved` document signal.
 
         Automaticaly check syntax when file is saved.
         """
-        controller.check_style(None, None, True)
+        self.controller.check_style(None, None, True)
