@@ -11,7 +11,13 @@ test suite.
 """
 
 import os
-
+from gi.repository import (
+    GObject,
+    Gdk,
+    Pango,
+    Gtk,
+    GtkSource,
+    )
 
 # Add additional Python lib paths as needed.
 paths = [
@@ -31,9 +37,6 @@ verbosity = 2
 
 
 if __name__ == '__main__':
-    gengedit = 'gen-gedit.py'
-    gengedit_path = 'utils/%s' % gengedit
-    os.spawnl(os.P_WAIT, gengedit_path, gengedit)
     params = vars()
     keys = params.keys()
     [params.pop(k) for k in keys if k.startswith('__')]
